clc;
close all;
clear all;
%==========================================================================
% Cuk Analysis
% Andrew Chuinard
% Tim Chairet
% xx/xx/09
%==========================================================================

%--------------------------------------------------------------------------
% Initial Parameters
%--------------------------------------------------------------------------
Vg    = 12;
Vo     = -24;
Vo_ripple = .05;
fs    = 100e3;     % Switching Frequency
Rl1    = 12;     %Load Resistance minimum
Rl2    = 120;    %load resistance maximum
Vm    = 5;         % PWM Conversion Ratio
Fm    = 1/Vm;
Vg_step = 20;
D      = 2/3;
DeltaV = abs(.05*Vo);


%----------------------------------------------------------------------
% Initial component design meeting requirements
%----------------------------------------------------------------------
 L2 = 2.2e-3;  % Educated Guess
 C2 =2.599e4*(1/fs)^2*Vg*D/(8*L2*DeltaV);   % Ripple Requirement
 % DCM Mode  2(L1||L2)/(Rts)<D'^2  So design for biggest R
 %  L1>L2*Rl2*(1/fs)*(1-D)^2/2/(L2-Rl2*(1/fs)*(1-D)^2/2);
 L1=L2*Rl2*(1/fs)*(1-D)^2/2/(L2-Rl2*(1/fs)*(1-D)^2/2);
 %No DVM Requirement C1>D^2Ts/(2Rl1)  smallest R
 C1 =20*D^2*(1/fs)/(2*Rl1);

f     = logspace(-2,8, 10e3);
s     = j*2*pi*f;
%--------------------------------------------------------------------------

%==========================================================================
% Calculations
%==========================================================================


    %----------------------------------------------------------------------
    % Case one:wz=wp1<<wp2
    %----------------------------------------------------------------------
    % Factorization Requirements #1:  C1>> D^2C2  C1 >> (L2/R^2)D^2
    fprintf('Case one:wz=wp1<<wp2\n');
    % Determine wz, wp1, wp2
    wz                  = sqrt((1-D)/(L1*C1));
    wp1                 = (1-D)/sqrt(L1*C1);
    wp2                 = 1/sqrt(L2*C2);
    %Keep L2 the same
    C2 = 1/((wp1*10)^2*L2);   % To satisfy wp1, this fails the ripple requirement
    % Determine wz, wp1, wp2
    wz                  = sqrt((1-D)/(L1*C1))
    wp1                 = (1-D)/sqrt(L1*C1)
    wp2                 = 1/sqrt(L2*C2)
    if C1 >= 10*D^2*C2
        fprintf('1st factorization approx met\n');
    else
        fprintf('1st factorization approx not met\n');
    end

    if C1 >= 10*(L2/Rl1^2)*D^2
        fprintf('2nd factorization approx met\n\n');
    else
        fprintf('2nd factorization approx not met\n\n');
    end

    %True Control to Output
    numeratorReal1 = s.^2*L1*C1/(1-D)-s*(D^2*L1/((1-D)^2*Rl1))+1;
    denominatorReal1    = s.^4*L1*C1*L2*C2/(1-D)^2+ ... 
                          s.^3*(L1*C1*L2/((1-D)^2*Rl1))+ ...
                          s.^2*(L1*C1/(1-D)^2+L2*C2+D^2/(1-D)^2*L1*C2)+ ...
                          s*(L2/Rl1+D^2/(1-D)^2*L1/Rl1)+1;
    Gvd_Real1           = numeratorReal1./denominatorReal1;

    Gvd_approx1         = numeratorReal1./((s.^2*L1*C1/(1-D)^2+s*D^2/(1-D)^2*L1/Rl1+1).* ...
                          (s.^2*L2*C2+s*L2/Rl1+1));
    %----------------------------------------------------------------------
    
    %----------------------------------------------------------------------
    % Case two:wz=wp1=wp2
    %----------------------------------------------------------------------
    % Determine L1,L2,C1,C2

    %Keep L2 the same
    C2 = 1/(wp1^2*L2);   % To satisfy wp1, this fails the ripple requirement 
     
    fprintf('Case two:wz=wp1=wp2\n');
    % Determine wz, wp1, wp2
    wz                  = sqrt((1-D)/(L1*C1))
    wp1                 = (1-D)/sqrt(L1*C1)
    wp2                 = 1/sqrt(L2*C2)
    % Factorization Requirements #1:  C1>> D^2C2  C1 >> (L2/R^2)D^2
    if C1 >= 10*D^2*C2
        fprintf('1st factorization approx met\n');
    else
        fprintf('1st factorization approx not met\n');
    end

    if C1 >= 10*(L2/Rl1^2)*D^2
        fprintf('2nd factorization approx met\n\n');
    else
        fprintf('2nd factorization approx not met\n\n');
    end

    %True Control to Output
    numeratorReal2 = s.^2*L1*C1/(1-D)-s*(D^2*L1/((1-D)^2*Rl1))+1;
    denominatorReal2    = s.^4*L1*C1*L2*C2/(1-D)^2+ ... 
                          s.^3*(L1*C1*L2/((1-D)^2*Rl1))+ ...
                          s.^2*(L1*C1/(1-D)^2+L2*C2+D^2/(1-D)^2*L1*C2)+ ...
                          s*(L2/Rl1+D^2/(1-D)^2*L1/Rl1)+1;
    Gvd_Real2          = numeratorReal2./denominatorReal2;

    Gvd_approx2         = numeratorReal2./((s.^2*L1*C1/(1-D)^2+s*D^2/(1-D)^2*L1/Rl1+1).* ...
                          (s.^2*L2*C2+s*L2/Rl1+1));
    %----------------------------------------------------------------------

    %----------------------------------------------------------------------
    % Case three:wz=wp1>>wp2
    %----------------------------------------------------------------------
    % Determine L1,L2,C1,C2

    %Keep L2 the same
    C2 = 1/((wp1/100)^2*L2);    

    fprintf('Case three:wz=wp1>>wp2\n');
    % Determine wz, wp1, wp2
    wz                  = sqrt((1-D)/(L1*C1))
    wp1                 = (1-D)/sqrt(L1*C1)
    wp2                 = 1/sqrt(L2*C2)
    if (1-D)^2 >= 2*(L1*L2)/(L1+L2)/Rl2*fs
        fprintf('DCM\n');
    else
        fprintf('CCM\n');
    end

    % Factorization Requirements #1:  C1>> D^2C2  C1 >> (L2/R^2)D^2
    if C1 >= 10*D^2*C2
        fprintf('1st factorization approx met\n');
    else
        fprintf('1st factorization approx not met\n');
    end

    if C1 >= 10*(L2/Rl1^2)*D^2
        fprintf('2nd factorization approx met\n');
    else
        fprintf('2nd factorization approx not met\n');
    end

    %True Control to Output
    numeratorReal3 = s.^2*L1*C1/(1-D)-s*(D^2*L1/((1-D)^2*Rl1))+1;
    denominatorReal3    = s.^4*L1*C1*L2*C2/(1-D)^2+ ... 
                          s.^3*(L1*C1*L2/((1-D)^2*Rl1))+ ...
                          s.^2*(L1*C1/(1-D)^2+L2*C2+D^2/(1-D)^2*L1*C2)+ ...
                          s*(L2/Rl1+D^2/(1-D)^2*L1/Rl1)+1;
    Gvd_Real3          = numeratorReal3./denominatorReal3;

    Gvd_approx3         = numeratorReal3./((s.^2*L1*C1/(1-D)^2+s*D^2/(1-D)^2*L1/Rl1+1).* ...
                          (s.^2*L2*C2+s*L2/Rl1+1));
    %----------------------------------------------------------------------    


%--------------------------------------------------------------------------
% Plots
%--------------------------------------------------------------------------
magnitudeGvd_Real1       = 20*log10(abs(Gvd_Real1)); 
phaseGvd_Real1            = unwrap(angle(Gvd_Real1))*180/pi;
magnitudeGvd_approx1         = 20*log10(abs(Gvd_approx1)); 
phaseGvd_approx1           = unwrap(angle(Gvd_approx1))*180/pi;

magnitudeGvd_Real2       = 20*log10(abs(Gvd_Real2)); 
phaseGvd_Real2            = unwrap(angle(Gvd_Real2))*180/pi;
magnitudeGvd_approx2        = 20*log10(abs(Gvd_approx2)); 
phaseGvd_approx2          = unwrap(angle(Gvd_approx2))*180/pi;

magnitudeGvd_Real3       = 20*log10(abs(Gvd_Real3)); 
phaseGvd_Real3            = unwrap(angle(Gvd_Real3))*180/pi;
magnitudeGvd_approx3        = 20*log10(abs(Gvd_approx3)); 
phaseGvd_approx3          = unwrap(angle(Gvd_approx3))*180/pi;

figure;
subplot(2,1,1);
plotHandle = semilogx(f,magnitudeGvd_Real1,f,magnitudeGvd_approx1);
box off;
grid on;
set(plotHandle(1),'color',[0 0 0.5]);  
set(plotHandle(2),'color',[0.5 0.5 1]); 
set(plotHandle(1),'linewidth',3);  
set(plotHandle(2),'linewidth',1.5); 
xlabel('Frequency (Hz)'); 
ylabel('$|H(2 \pi f)|$ (dB)'); 
title('CUK Case one:wz=wp1<<wp2'); 
legend('true','approx');
set(get(gca,'ylabel'),'interpreter','LaTeX')

subplot(2,1,2);
    plotHandle = semilogx(f,phaseGvd_Real1,f,phaseGvd_approx1);
    box off;
    grid on;
    set(plotHandle(1),'color',[0 0 0.5]);
    set(plotHandle(2),'color',[0.5 0.5 1]);
    set(plotHandle(1),'linewidth',3);
    set(plotHandle(2),'linewidth',1.5);
    xlabel('Frequency (Hz)');
    ylabel('$\phi(2 \pi f)$ (degrees)');
    legend('true','approx');
    set(get(gca,'ylabel'),'interpreter','LaTeX'); 
    
figure;
subplot(2,1,1);
plotHandle = semilogx(f,magnitudeGvd_Real2,f,magnitudeGvd_approx2);
box off;
grid on;
set(plotHandle(1),'color',[0 0 0.5]);  
set(plotHandle(2),'color',[0.5 0.5 1]); 
set(plotHandle(1),'linewidth',3);  
set(plotHandle(2),'linewidth',1.5); 
xlabel('Frequency (Hz)'); 
ylabel('$|H(2 \pi f)|$ (dB)'); 
title('CUK Case two:wz=wp1=wp2'); 
legend('true','approx');
set(get(gca,'ylabel'),'interpreter','LaTeX')

subplot(2,1,2);
    plotHandle = semilogx(f,phaseGvd_Real2,f,phaseGvd_approx2);
    box off;
    grid on;
    set(plotHandle(1),'color',[0 0 0.5]);
    set(plotHandle(2),'color',[0.5 0.5 1]);
    set(plotHandle(1),'linewidth',3);
    set(plotHandle(2),'linewidth',1.5);
    xlabel('Frequency (Hz)');
    ylabel('$\phi(2 \pi f)$ (degrees)');
    legend('true','approx');
    set(get(gca,'ylabel'),'interpreter','LaTeX'); 

figure;
subplot(2,1,1);
plotHandle = semilogx(f,magnitudeGvd_Real3,f,magnitudeGvd_approx3);
box off;
grid on;
set(plotHandle(1),'color',[0 0 0.5]);  
set(plotHandle(2),'color',[0.5 0.5 1]); 
set(plotHandle(1),'linewidth',3);  
set(plotHandle(2),'linewidth',1.5); 
xlabel('Frequency (Hz)'); 
ylabel('$|H(2 \pi f)|$ (dB)'); 
title('CUK Case three:wz=wp1>>wp2'); 
legend('true','approx');
set(get(gca,'ylabel'),'interpreter','LaTeX')

subplot(2,1,2);
    plotHandle = semilogx(f,phaseGvd_Real3,f,phaseGvd_approx3);
    box off;
    grid on;
    set(plotHandle(1),'color',[0 0 0.5]);
    set(plotHandle(2),'color',[0.5 0.5 1]);
    set(plotHandle(1),'linewidth',3);
    set(plotHandle(2),'linewidth',1.5);
    xlabel('Frequency (Hz)');
    ylabel('$\phi(2 \pi f)$ (degrees)');
    legend('true','approx');
    set(get(gca,'ylabel'),'interpreter','LaTeX'); 


