close all;
clear all;
%==========================================================================
% 2nd Cuk Factorization Lossless Damping Case
% Andrew Chuinard
% Tim Chairet
% 05/24/09
%==========================================================================

%--------------------------------------------------------------------------
% Initial Parameters
%--------------------------------------------------------------------------
Vg    = 12;
Vo     = -24;
Vo_ripple = .05;
fs    = 100e3;     % Switching Frequency
Rl1    = 12;     %Load Resistance minimum
Rl2    = 120;    %load resistance maximum
Vm    = 5;         % PWM Conversion Ratio
Fm    = 1/Vm;
Vg_step = 20;
DeltaV = abs(.05*Vo);

%Damping
C = 2.3e-3;
r = 1;
D      = 2/3;

s = tf('s');
%--------------------------------------------------------------------------

%==========================================================================
% Calculations
%==========================================================================


    %----------------------------------------------------------------------
    % wz=wp1>wp2
    %----------------------------------------------------------------------
   

 L2 = 2.2e-3;  % Educated Guess
 C2 =2.599e4*(1/fs)^2*Vg*D/(8*L2*DeltaV);   % Ripple Requirement
 % DCM Mode  2(L1||L2)/(Rts)<D'^2  So design for biggest R
 %  L1>L2*Rl2*(1/fs)*(1-D)^2/2/(L2-Rl2*(1/fs)*(1-D)^2/2);
 L1=L2*Rl2*(1/fs)*(1-D)^2/2/(L2-Rl2*(1/fs)*(1-D)^2/2);
 %No DVM Requirement C1>D^2Ts/(2Rl1)  smallest R
 C1 =20*D^2*(1/fs)/(2*Rl1);
 % Determine wz, wp1, wp2
 wz                  = sqrt((1-D)/(L1*C1));
 wp1                 = (1-D)/sqrt(L1*C1);
 wp2                 = 1/sqrt(L2*C2);
    %Ripple Criteria 
    if DeltaV >= (1/fs)^2*Vg*D/(8*L2*C2)
        fprintf('Ripple Criteria MET\n');
    else
        fprintf('Ripple Criteria NOT MET\n');
    end

    %CCM Criteria
    if (1-D)^2 >= 2*(L1*L2)/(L1+L2)/Rl2*fs
        fprintf('DCM \n');
    else
        fprintf('CCM \n');
    end

        %DVM Criteria 
    if C1 >= D^2*(1/fs)/(2*Rl1);
        fprintf('CVM\n');
    else
        fprintf('DVM\n');
    end
    
    % Factorization Requirements 
    if L2 >= 7.5*D^2/(1-D)^2*L1
        fprintf('Factorization approx met \n');
    else
        fprintf('Factorization approx not met\n');
    end


    % Lossless Factorization Requirements 
    if L1 <= 10*(1-D)^2*r^2*C
        fprintf('Lossless Factorization approx met \n');
    else
        fprintf('Lossless Factorization approx not met\n');
    end

    % Print output ripple values
    i1Ripple = Vg/L1*D*(1/fs)
    i2Ripple = Vg/L2*D*(1/fs)
    V1Ripple = Vg*(1/fs)/(Rl1*C1)*(D/(1-D))^2
    V2Ripple = (1/fs)^2*Vg*D/(8*L2*C2)
    I1 = Vg/Rl2*(D/(1-D))^2
    I2 = Vg/Rl2*(D/(1-D))
    V1 = Vg/(1-D)
    V2 = Vg*(D/(1-D))

    
    % Control to Output
    numeratorReal = s^2*L1*C1/(1-D)-s*(D^2*L1/((1-D)^2*Rl2))+1;

    denominatorReal    = s^4*L1*C1*L2*C2/(1-D)^2+ ... 
                          s^3*(L1*C1*L2/((1-D)^2*Rl2))+ ...
                          s^2*(L1*C1/(1-D)^2+L2*C2+D^2/(1-D)^2*L1*C2)+ ...
                          s*(L2/Rl2+D^2/(1-D)^2*L1/Rl2)+1;                  
    Gvd_Real          = numeratorReal/denominatorReal;
    
   
    Gvd_approx         = numeratorReal/((s^2*L1*C1/(1-D)^2+s*L1/((1-D)^2*r)+1)* ...
                          (s^2*L2*C2+s*L2/Rl2+1));
    
    %----------------------------------------------------------------------    
    
    
    %----------------------------------------------------------------------
    % lossless control to output
    %----------------------------------------------------------------------
    
    A1 = [0 0 0 0 0; ...
      0 0 (1/L2) (-1/L2) 0; ...
      0 (-1/C1) (-1/(r*C1)) 0 (1/(r*C1)); ...
      0 (1/C2)  0 (-1/(Rl2*C2)) 0; ...
      0 0 (1/(r*C)) 0 (-1/(r*C))];
    A2 = [0    0     -1/L1     0         0; ...
          0    0     0         -1/L2     0; ...
          1/C1 0     -1/(r*C1) 0         1/(r*C1); ...
          0    1/C2  0         -1/(Rl2*C2)  0; ...
          0    0     1/(r*C)   0         -1/(r*C);];
    B1 = [1/L1 ; 0 ; 0 ; 0 ; 0];
    B2 = [1/L1 ; 0 ; 0 ; 0 ; 0];
    Cm1 = [0 0 0 1 0];
    Cm2 = Cm1;
    E1 = [0];
    E2 = E1;
    A =  simplify(D*A1 + (1-D)*A2); 
    B =  simplify(D*B1 + (1-D)*B2);
    Cm =  simplify(D*Cm1 + (1-D)*Cm2);
    E =  simplify(D*E1 + (1-D)*E2);

    X = -A^(-1)*B*Vg;

    Bd = (A1-A2)*X + (B1-B2)*Vg;
    Ed = (Cm1-Cm2)*X + (E1-E2)*Vg;
    sI = [s 0 0 0 0; ...
          0 s 0 0 0; ...
          0 0 s 0 0; ...
          0 0 0 s 0;
          0 0 0 0 s];

  sys = ss(A,B,Cm,E);
  V_G = tf(sys);
  sys2 = ss(A,Bd,Cm,Ed);
  V_D1 = tf(sys2); 
  V_D = Cm*(sI-A)^(-1)*Bd + Ed;

% Compensator
Gcon = 10/s*(s/100+1)*(s/100+1);


%--------------------------------------------------------------------------
% Plots
%--------------------------------------------------------------------------
figure;
bode(Fm*V_D*Gcon,Fm*Gvd_Real*Gcon);
legend('True-Lossless Damping','True-No Damping');

figure;
 bode(V_G/(1+Gcon*Fm*V_D));
 title('Closed Loop Audiosusceptability')
figure;
margin(Fm*V_D*Gcon);