close all;
clear all;
%==========================================================================
% 2nd Cuk Factorization
% Andrew Chuinard
% Tim Chairet
% xx/xx/09
%==========================================================================

%--------------------------------------------------------------------------
% Initial Parameters
%--------------------------------------------------------------------------
Vg    = 12;
Vo     = -24;
Vo_ripple = .05;
fs    = 100e3;     % Switching Frequency
Rl1    = 12;     %Load Resistance minimum
Rl2    = 120;    %load resistance maximum
Vm    = 5;         % PWM Conversion Ratio
Fm    = 1/Vm;
Vg_step = 20;
DeltaV = abs(.05*Vo);

%Parasitics
rl1 = 100e-3;
rl2 = 250e-3;
rc1 = 1e-3;
rc2 = 50e-3;

D      = 2/3;
DeltaV = abs(.05*Vo);

s = tf('s');


%==========================================================================
% Calculations
%==========================================================================



 %----------------------------------------------------------------------
 % wz=wp1>wp2
 %----------------------------------------------------------------------
   

 L2 = 2.2e-3;  % Educated Guess
 C2 =2.599e4*(1/fs)^2*Vg*D/(8*L2*DeltaV);   % Ripple Requirement
 % DCM Mode  2(L1||L2)/(Rts)<D'^2  So design for biggest R
 %  L1>L2*Rl2*(1/fs)*(1-D)^2/2/(L2-Rl2*(1/fs)*(1-D)^2/2);
 L1=L2*Rl2*(1/fs)*(1-D)^2/2/(L2-Rl2*(1/fs)*(1-D)^2/2);
 %No DVM Requirement C1>D^2Ts/(2Rl1)  smallest R
 C1 =20*D^2*(1/fs)/(2*Rl1);

    % Determine wz, wp1, wp2
wz                  = sqrt((1-D)/(L1*C1));
wp1                 = (1-D)/sqrt(L1*C1);
wp2                 = 1/sqrt(L2*C2);
    
    
      
   %Ripple Criteria 
    if DeltaV >= (1/fs)^2*Vg*D/(8*L2*C2)
        fprintf('Ripple Criteria MET\n');
    else
        fprintf('Ripple Criteria NOT MET\n');
    end

    %CCM Criteria
    if (1-D)^2 >= 2*(L1*L2)/(L1+L2)/Rl2*fs
        fprintf('DCM \n');
    else
        fprintf('CCM \n');
    end

        %DVM Criteria 
    if C1 >= D^2*(1/fs)/(2*Rl1);
        fprintf('CVM\n');
    else
        fprintf('DVM\n');
    end
    
    % Factorization Requirements 
    if L2 >= 7.5*D^2/(1-D)^2*L1
        fprintf('Factorization approx met \n');
    else
        fprintf('Factorization approx not met\n');
    end


    % Print output ripple values
    i1Ripple = Vg/L1*D*(1/fs)
    i2Ripple = Vg/L2*D*(1/fs)
    V1Ripple = Vg*(1/fs)/(Rl1*C1)*(D/(1-D))^2
    V2Ripple = (1/fs)^2*Vg*D/(8*L2*C2)
    I1 = Vg/Rl2*(D/(1-D))^2
    I2 = Vg/Rl2*(D/(1-D))
    V1 = Vg/(1-D)
    V2 = Vg*(D/(1-D))





    %True Control to Output
    numeratorReal = s^2*L1*C1/(1-D)-s*(D^2*L1/((1-D)^2*Rl1))+1;

    denominatorReal    = s^4*L1*C1*L2*C2/(1-D)^2+ ... 
                          s^3*(L1*C1*L2/((1-D)^2*Rl1))+ ...
                          s^2*(L1*C1/(1-D)^2+L2*C2+D^2/(1-D)^2*L1*C2)+ ...
                          s*(L2/Rl1+D^2/(1-D)^2*L1/Rl1)+1;
    Gvd_Real          = numeratorReal/denominatorReal;
 
    
    Qp2 =1/(L2/Rl1*wp2);
    %----------------------------------------------------------------------    
A1 = [-(rl1/L1) 0 0 0 ; ...
      0 -(rc1+rl2+rc2*Rl1/(rc2+Rl1))/L2 (1/L2) (rc2/(Rl1+rc2)-1)/L2; ...
      0 (-1/C1) 0 0 ; ...
      0 (Rl1/(rc2+Rl1)/C2) 0 -(1/(rc2+Rl1)/C2)];

A2 = [-(rl1+rc1/L1) 0 -(1/L1) 0 ; ...
      0 -(rl2+rc2*Rl1/(rc2+Rl1))/L2 0 (rc2/(Rl1+rc2)-1)/L2; ...
      (1/C1) 0 0 0 ; ...
      0 (Rl1/(rc2+Rl1)/C2) 0 -(1/(rc2+Rl1)/C2)];


B1 = [1/L1 ; 0 ; 0 ; 0];
B2 = [1/L1 ; 0 ; 0 ; 0];

Cm1 = [0 0 0 1];
Cm2 = Cm1;

E1 = [0];
E2 = E1;


A =  simplify(D*A1 + (1-D)*A2); 
B =  simplify(D*B1 + (1-D)*B2);
Cm =  simplify(D*Cm1 + (1-D)*Cm2);
E =  simplify(D*E1 + (1-D)*E2);


X = -A^(-1)*B*Vg;

Bd = (A1-A2)*X + (B1-B2)*Vg;
Ed = (Cm1-Cm2)*X + (E1-E2)*Vg;
sI = [s 0 0 0; ...
      0 s 0 0; ...
      0 0 s 0; ...
      0 0 0 s];
V_D = Cm*(sI-A)^(-1)*Bd + Ed;

sys = ss(A,B,Cm,E);
  V_G = tf(sys);
  sys2 = ss(A,Bd,Cm,Ed);
  V_D1 = tf(sys2); 
% Compensator
Gcon = 10/s*(s/100+1)*(s/100+1);



    
%--------------------------------------------------------------------------
% Plots
%--------------------------------------------------------------------------
figure;
bode(V_D, Gvd_Real*100);
legend('True Parasitics','True-No Parasitics','k');
figure;
margin(Gcon*Fm*V_D);

figure;
margin(V_G/(1+Gcon*Fm*V_D));
title('Closed Loop Audiosusceptability')